from __future__ import division, with_statement, print_function

import pygame
from level import Level

class Animal:
    ANIM_LENGTH = 0.05 # animation frame length in percentage of a complete tile step
    IMAGES = (
        ('media/tiles/animal_giraffe.png','media/tiles/animal_giraffe_1.png','media/tiles/animal_giraffe_2.png',),
        ('media/tiles/animal_bunny.png','media/tiles/animal_bunny_1.png','media/tiles/animal_bunny_2.png',),
        ('media/tiles/animal_fox.png','media/tiles/animal_fox_1.png','media/tiles/animal_fox_2.png',),
        ('media/tiles/animal_frog.png','media/tiles/animal_frog_1.png','media/tiles/animal_frog_2.png',),
        ('media/tiles/animal_pelican.png','media/tiles/animal_pelican_1.png','media/tiles/animal_pelican_2.png',),
        )
        
    def __init__(self, game, level, type, x, y, dir):
        self.game = game
        self.level = level
        self.type = type
        self.set_direction(dir)
        self.curImage = 0        
        self.linked = None
        self.moving = False
        self.tilex = self.oldx = self.nextx = x
        self.tiley = self.oldy = self.nexty = y
        self.posx, self.posy = self.oldposx, self.oldposy = self.newposx, self.newposy = self.level.get_tile_coords(self.tilex, self.tiley)
        self.prevpct = -1
        self.show = True
        
        
    def reset(self, x, y, dir):
        self.tilex = self.oldx = self.nextx = x
        self.tiley = self.oldy = self.nexty = y
        self.curImage = 0
        self.posx, self.posy = self.level.get_tile_coords(self.tilex, self.tiley)
        self.set_direction(dir)
        self.linked = None
        self.moving = False
        self.show = True


    def set_direction(self, dir):
        # flip/rotation functions for either direction
        transforms = [lambda img: pygame.transform.flip(img, True, False), lambda img: img, 
            lambda img: pygame.transform.rotate(img, 90), 
            lambda img: pygame.transform.rotate(img, -90)]
        self.dir = dir
        self.images = []
        for image in Animal.IMAGES[self.type]:
            self.images.append(transforms[dir](
                pygame.transform.smoothscale(
                    pygame.image.load(image), (Level.TILESIZE, Level.TILESIZE))))
        
        
    def render(self, display):
        if self.show:
            display.blit(self.images[self.curImage], (self.posx, self.posy))
        
        
    def update(self, pct):
        if not self.moving: return
        self.tilex, self.tiley = self.nextx, self.nexty
        
        # pct is the percentage between old and new position we should display
        if self.dir == 0:
            dx = -1
            dy = 0
        elif self.dir == 1:
            dx = 1
            dy = 0
        elif self.dir == 2:
            dx = 0
            dy = -1
        elif self.dir == 3:
            dx = 0
            dy = 1
            
        if pct >= 0.5:
            val = int(Level.TILESIZE * (1-pct))
            self.posx = self.newposx - dx*val
            self.posy = self.newposy - dy*val
        else:
            val = int(Level.TILESIZE * pct)
            self.posx = self.oldposx + dx*val
            self.posy = self.oldposy + dy*val
            
        # update animation
        #animStep = int(pct / Animal.ANIM_LENGTH)
        #self.curImage = animStep % len(self.images)
        if pct != self.prevpct:
            self.curImage = (self.curImage+1) % len(self.images)
            self.prevpct = pct
            if pct == 0:
                self.curImage = 0
        
        
        
    def calc_next_tile(self):
        self.moving = False
        self.oldx = self.tilex
        self.oldy = self.tiley
        self.oldposx, self.oldposy = self.level.get_tile_coords(self.oldx, self.oldy)
        if self.linked != None: return

        cur = self.level.get_tile(self.tilex, self.tiley)
        if cur >= Level.ARROW_LEFT:
            self.set_direction(cur - Level.ARROW_LEFT)

        for i in range(4):
            x,y = self.level.get_neighbour(self.tilex, self.tiley, self.dir)
            other = self.game.get_occupant(x,y)
            if other != None and other.type != self.type:
                newdir = (2,3,1,0)[self.dir] # turn to the right
                self.set_direction(newdir)
            else: break
                
        other = self.game.get_occupant(x,y)
        if self.level.get_tile(x,y) == Level.BLOCKED or (other != None and other.type != self.type):
            self.moving = False
            self.curImage = 0
        else:
            self.moving = True
            self.nextx, self.nexty = x, y
            self.newposx, self.newposy = self.level.get_tile_coords(x, y)
        if other and other.type == self.type:
            # moving to link with pair
            self.linked = other
            other.linked = self
            other.nextx, other.nexty = x, y
            other.newposx, other.newposy = self.level.get_tile_coords(x, y)
            if other.nextx == other.tilex and other.nexty == other.tiley:
                other.moving = False
                other.curImage = 0
            else:
                other.moving = True
