from __future__ import division, with_statement, print_function

import pygame
import random
from level import Level
from interface import Interface, AnimatedSprite
from animal import Animal

class Game:
    STEPTIME = 1000
    SCROLLSPEED = 0.7
    MUSIC = ('media/sound/music1.ogg',)
    MUSIC_ENDED = pygame.USEREVENT+1
    MATING_SOUNDS = ('media/sound/mating_0.ogg', 'media/sound/mating_1.ogg')
    SPRITE_WON = ('media/gui/win_overlay.png',)
    SPRITES_MATING = (
        ('media/tiles/pair_giraffe_0.png','media/tiles/pair_giraffe_1.png','media/tiles/pair_giraffe_2.png',),
        ('media/tiles/pair_bunny_0.png','media/tiles/pair_bunny_1.png','media/tiles/pair_bunny_2.png',),
        ('media/tiles/pair_fox_0.png','media/tiles/pair_fox_1.png','media/tiles/pair_fox_2.png',),
        ('media/tiles/pair_frog_0.png','media/tiles/pair_frog_1.png','media/tiles/pair_frog_2.png',),
        ('media/tiles/pair_pelican_0.png','media/tiles/pair_pelican_1.png','media/tiles/pair_pelican_2.png',),
    )
    
    def __init__(self):
        self._continue = False
        self.size = self.width, self.height = 1024, 768
        self.selectedArrow = 4 # arrow to be placed in the level
        self.animals = []
        self.simulate = False
        self.timeToStep = 0
        self.scrollx = 0
        self.scrolly = 0
        self.levelWon = False
        self.lastMusicReset = 0
        self.animated = []
        self.background = pygame.image.load('media/gui/game_background.png')
        

    def initialise(self):
        pygame.init()
        pygame.font.init()
        #pygame.mixer.pre_init(44100,-16,2,1024)
        pygame.mixer.init()
        self._display = pygame.display.set_mode(self.size, pygame.HWSURFACE | pygame.DOUBLEBUF)
        pygame.display.set_caption("Animal mating rituals")
        self.interface = Interface(self, 0, 628, 1024, 140)
        self.level = Level(self, 0, 0, 1024, 628)
        self._continue = True
        #self.imageWon = pygame.image.load('media/gui/win_overlay.png')
        

    def shutdown(self):
        pygame.mixer.quit()
        pygame.font.quit()
        pygame.quit()
        
        
    def get_occupant(self, x, y):
        for animal in self.animals:
            if (animal.tilex == x and animal.tiley == y) or (animal.nextx == x and animal.nexty == y):
                return animal
        return None
        

    def process_event(self, event):
        if event.type == Game.MUSIC_ENDED:
            self.play_music()
        elif event.type == pygame.QUIT:
            self._continue = False
        elif self.levelWon and event.type in (pygame.MOUSEBUTTONDOWN, pygame.KEYDOWN):
            self._continue = False
        elif self.levelWon:
            return
        elif event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 1:
                self.interface.mouse_down(event.pos)
                if not self.simulate:
                    self.level.mouse_down(event.pos, self.selectedArrow)
            elif event.button == 3:
                # cancel current action by right button
                self.selectedArrow = 4
                self.interface.unselect()
        elif event.type == pygame.MOUSEMOTION and not self.simulate:
            self.level.mouse_moved(event.pos)
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE:
                self._continue = False
            elif event.key == pygame.K_LEFT:
                self.scrollx -= 1
            elif event.key == pygame.K_RIGHT:
                self.scrollx += 1
            elif event.key == pygame.K_UP:
                self.scrolly -= 1
            elif event.key == pygame.K_DOWN:
                self.scrolly += 1
        elif event.type == pygame.KEYUP:
            if event.key == pygame.K_LEFT:
                self.scrollx += 1
            elif event.key == pygame.K_RIGHT:
                self.scrollx -= 1
            elif event.key == pygame.K_UP:
                self.scrolly += 1
            elif event.key == pygame.K_DOWN:
                self.scrolly -= 1
            

    def update(self, time):
        if (self.scrollx != 0 or self.scrolly != 0) and not self.levelWon:
            self.level.offset[0] -= self.scrollx * Game.SCROLLSPEED*time
            self.level.offset[1] -= self.scrolly * Game.SCROLLSPEED*time
            for animal in self.animals:
                animal.posx -= self.scrollx * Game.SCROLLSPEED*time
                animal.posy -= self.scrolly * Game.SCROLLSPEED*time
                animal.oldposx -= self.scrollx * Game.SCROLLSPEED*time
                animal.oldposy -= self.scrolly * Game.SCROLLSPEED*time
                animal.newposx -= self.scrollx * Game.SCROLLSPEED*time
                animal.newposy -= self.scrolly * Game.SCROLLSPEED*time
            for anim in self.animated:
                anim.pos[0] -= self.scrollx * Game.SCROLLSPEED*time
                anim.pos[1] -= self.scrolly * Game.SCROLLSPEED*time
                
        for anim in self.animated:
            anim.update(time)
                
        if not self.levelWon:
            self.timeElapsed += time/1000
            self.interface.set_time(self.timeElapsed)
                
        if not self.simulate or self.levelWon: return
        self.timeToStep += time
        if self.timeToStep >= Game.STEPTIME:
            # check if all animals are linked
            allLinked = True
            for animal in self.animals:
                if animal.linked == None:
                    allLinked = False
                else:
                    animal.show = False
                    linkPos = (animal.nextx, animal.nexty)
                    if linkPos not in self.matings:
                        self.matings.add(linkPos)
                        sound = pygame.mixer.Sound(random.choice(Game.MATING_SOUNDS))
                        sound.play()
                        self.animated.append(AnimatedSprite(Game.SPRITES_MATING[animal.type], (animal.posx,animal.posy), 250, Level.TILESIZE))
            if allLinked:
                self.levelWon = True
                self.simulate = False
                self.animated.append(AnimatedSprite(Game.SPRITE_WON, (384,186), 250))
                return
                
            # choose animals for next tile
            for animal in sorted(self.animals, key=lambda a: a.tiley*self.level.width+a.tilex):
                animal.calc_next_tile()
            for animal in self.animals:
                animal.update(0)
            self.timeToStep = 0
        else:
            pct = round(self.timeToStep / Game.STEPTIME * 3) / 3
            for animal in self.animals:
                animal.update(pct)
        

    def render(self):
        self._display.blit(self.background, (0,0))
        self.level.render(self._display)
        for animal in self.animals:
            animal.render(self._display)
        for anim in self.animated:
            anim.render(self._display)
        self.interface.render(self._display)
        #if self.levelWon:
        #    self._display.blit(self.imageWon, (384, 186))
        pygame.display.flip()
        
        
    def set_arrow(self, arrow):
        "Change the selected arrow to place in the level."
        self.selectedArrow = arrow
        
        
    def update_arrows(self, numArrows):
        "Update the amount of placeable arrows left."
        self.interface.update_arrows(numArrows)
        
        
    def clear_grid(self):
        "Remove all user-placed arrows from the game grid."
        self.level.clear()
        
        
    def load_level(self, file):
        self.level.load_from_file(file)
        self.levelWon = False
        self._continue = True
        self.simulate = False
        self.timeElapsed = 0
        # create animals
        self.animals = []
        for a in self.level.animals:
            self.animals.append(Animal(self, self.level, a[0], a[1], a[2], a[3]))
            
            
    def play_music(self):
        # play a randomly chosen song
        if self.setMusic:
            self.setMusic = False
            return
                    
        file = random.choice(Game.MUSIC)
        if not pygame.mixer.music.get_busy():
            pygame.mixer.music.load(file)
            pygame.mixer.music.play()
        else:
            self.setMusic = True
            pygame.mixer.music.fadeout(1000)
            pygame.mixer.music.queue(file)
        pygame.mixer.music.set_endevent(Game.MUSIC_ENDED)
        
        
    def run(self, levelFile):
        self.load_level(levelFile)
        self.interface.reset()
        self.animated = []
        self.setMusic = False
        self.play_music()
        
        clock = pygame.time.Clock()
        clock.tick()
        
        while self._continue:
            clock.tick(120)
            for event in pygame.event.get():
                self.process_event(event)
            self.update(clock.get_time())
            self.render()
            
        return self.levelWon
            
            
    def start(self):
        "Start simulation of scenario."
        self.simulate = True
        self.timeToStep = Game.STEPTIME
        self.matings = set()
        
       
    def stop(self):
        "Stop simulation of scenario and reset animals."
        self.simulate = False
        self.animated = []
        # reset animals
        for i in range(len(self.animals)):
            a = self.level.animals[i]
            self.animals[i].reset(a[1], a[2], a[3])


# if started as main file, run the level specified as parameter
if __name__ == "__main__":
    import sys
    if len(sys.argv) != 2:
        print("Usage: {0} levelfile".format(sys.argv[0]))
        sys.exit(0)
        
    game = Game()
    try:
        game.initialise()
        game.run(sys.argv[1])
    #except Exception as e:
    #    print("Uncaught exception:", e)
    finally:    
        game.shutdown()
