from __future__ import division, with_statement, print_function

import pygame

class Button:
    HIGHLIGHT = (150,0,0)
    
    def __init__(self, image, pos, text=''):
        "Construct a clickable button from the given image at the given position"
        self.set_image(image)
        self.pos = pos
        self.set_text(text)
        self.sound = pygame.mixer.Sound('media/sound/menu_button.ogg')
        self.highlight = False


    def set_text(self, text):
        # render text
        font = pygame.font.Font('media/freesansbold.ttf', 18)
        self.text = font.render(text, True, (255,255,255))
        
        
    def set_image(self, image):
        self.image = pygame.image.load(image)
        self.size = self.image.get_size()
        
        
    def set_highlight(self, highlight):
        self.highlight = highlight


    def render(self, display):
        display.blit(self.image, self.pos)
        if self.highlight:
            pygame.draw.rect(display, Button.HIGHLIGHT, 
                self.image.get_rect().move(self.pos[0],self.pos[1]).inflate(2,2), 2)
        textx = self.pos[0] + self.image.get_width()//2
        textx = textx - self.text.get_width()//2
        texty = self.pos[1] + self.image.get_height() + 5
        display.blit(self.text, (textx, texty))


    def is_clicked(self, mousex, mousey):
        if (mousex >= self.pos[0] and mousex <= self.size[0]+self.pos[0]
            and mousey >= self.pos[1] and mousey <= self.size[1]+self.pos[1]):
            self.sound.play()
            return True
        return False
        
        
        
class AnimatedSprite:
    def __init__(self, images, pos, animLength, size=0):
        self.images = []
        for image in images:
            self.images.append(pygame.image.load(image))
        if size != 0:
            for i in range(len(self.images)):
                self.images[i] = pygame.transform.smoothscale(self.images[i], (size,size))
        self.pos = list(pos)
        self.curImage = 0
        self.time = 0
        self.length = animLength
        
        
    def update(self, time):
        self.time += time
        if self.time >= self.length:
            self.time -= self.length
            self.curImage = (self.curImage+1) % len(self.images)
            
            
    def render(self, display):
        display.blit(self.images[self.curImage], self.pos)
        



class Interface:
    def __init__(self, game, x, y, w, h):
        self.game = game
        self.renderRect = (x, y, w, h)
        self.background = pygame.image.load('media/gui/gui_background.png')
        self.arrows = []
        self.arrows.append(Button('media/gui/button_left.png', (x+20, y+25)))
        self.arrows.append(Button('media/gui/button_right.png', (x+100, y+25)))
        self.arrows.append(Button('media/gui/button_up.png', (x+180, y+25)))
        self.arrows.append(Button('media/gui/button_down.png', (x+260, y+25)))
        self.arrows.append(Button('media/gui/button_none.png', (x+340, y+25)))
        self.arrows[-1].set_highlight(True)
        self.startStop = Button('media/gui/button_start.png', (x+740, y+30))
        self.clear = Button('media/gui/button_clear.png', (x+880, y+30))
        self.isStarted = False
        self.time = 0
        
        
    def reset(self):
        self.time = 0
        self.startStop.set_image('media/gui/button_start.png')
        self.isStarted = False


    def render(self, display):
        #pygame.draw.rect(display, Interface.BORDER, self.renderRect)
        #pygame.draw.rect(display, Interface.BACKGROUND, pygame.Rect(self.renderRect).inflate(-4,-4))
        display.blit(self.background, self.renderRect)
        for arrow in self.arrows:
            arrow.render(display)
        self.startStop.render(display)
        self.clear.render(display)
        # display elapsed time
        minutes = int(self.time) // 60
        seconds = int(self.time) % 60
        font = pygame.font.Font('media/freesansbold.ttf', 53)
        surface = font.render("{0}:{1:02}".format(minutes, seconds), True, (255,255,255))
        posx = self.renderRect[0] + 600
        posy = self.renderRect[1] + 35
        display.blit(surface, (posx,posy))
        
        
    def mouse_down(self, pos):
        arrowPressed = -1
        for i in range(len(self.arrows)):
            if self.arrows[i].is_clicked(pos[0], pos[1]):      
                arrowPressed = i
        if arrowPressed != -1:
            for arrow in self.arrows:
                arrow.set_highlight(False)
            self.arrows[arrowPressed].set_highlight(True)
            self.game.set_arrow(arrowPressed)
            
        if self.clear.is_clicked(pos[0], pos[1]):
            if self.isStarted:
                self.game.stop()
                self.startStop.set_image('media/gui/button_start.png')
                self.isStarted = False
            self.game.clear_grid()
            
        if self.startStop.is_clicked(pos[0], pos[1]):
            if self.isStarted:
                self.game.stop()
                self.startStop.set_image('media/gui/button_start.png')
                self.isStarted = False
            else:
                self.game.start()
                self.startStop.set_image('media/gui/button_stop.png')
                self.isStarted = True
                
                
    def set_time(self, time):
        self.time = time
            
    
    def unselect(self):
        for arrow in self.arrows:
            arrow.set_highlight(False)
        self.arrows[-1].set_highlight(True)
            
            
    def update_arrows(self, numArrows):
        for i in range(len(numArrows)):
            self.arrows[i].set_text("x{0}".format(numArrows[i]))
