import pygame
import copy
import hashlib
import random

class Level:
    # tile status codes
    EMPTY = -1 # not part of the actual level
    PASSABLE = 0 # animals can walk on this tile
    BLOCKED = 1 # animals can't walk, will stop in front of this tile
    ARROW_LEFT = 2 # animals go left when entering this tile
    ARROW_RIGHT = 3 # animals go right when entering this tile
    ARROW_UP = 4 # animals go up when entering this tile
    ARROW_DOWN = 5 # animals go down when entering this tile
    IMAGES = ('media/tiles/background/set_0/passable_0.png', 'media/tiles/background/set_0/impassable_0.png',
        'media/tiles/background/set_0/left.png', 'media/tiles/background/set_0/right.png', 'media/tiles/background/set_0/up.png', 'media/tiles/background/set_0/down.png')
    TILESETS = ('set_0','set_1')
    TILES = (
        ('passable_0.png', 'passable_1.png', 'passable_2.png', 'passable_3.png'),
        ('impassable_0.png', 'impassable_1.png', 'impassable_2.png', 'impassable_3.png'),
        ('left.png','left.png','left.png','left.png',),
        ('right.png','right.png','right.png','right.png',),
        ('up.png','up.png','up.png','up.png',),
        ('down.png','down.png','down.png','down.png',)
    )
    # tile size in pixels
    TILESIZE = 64

    
    def __init__(self, game, x, y, w, h):
        self.game = game
        self.width = 0
        self.height = 0
        self.grid = [[]]
        self.origGrid = [[]]
        self.numArrows = [0,0,0,0] # number of arrows the player can place
        self.title = ""
        self.description = ""
        self.highlightX = -1
        self.highlightY = -1
        # images for tiles w/o animals
        self.images = []
        # initial render area
        self.renderRect = (x, y, w, h)
        # rendering offset, used for scrolling
        self.offset = [0, 0]
        # sound to play when arrow is placed/removed
        self.sound = pygame.mixer.Sound('media/sound/place_arrow.ogg')
        
        
    def load_tile_set(self):
        "randomly load one of the available tilesets."
        tileset = random.choice(Level.TILESETS)
        self.images = []
        for imgList in Level.TILES:
            images = []
            for image in imgList:
                images.append(pygame.transform.smoothscale(
                    pygame.image.load('media/tiles/background/'+tileset+'/'+image), (Level.TILESIZE, Level.TILESIZE)))
            self.images.append(images)
        # assign a random number to each field in the grid
        self.rand = [[random.randint(0,3) for j in range(self.width)] for i in range(self.height)]
                


    def load_from_file(self, filename):
        with open(filename, 'r') as f:
            # read dimensions of level grid
            dims = f.readline().split(' ')
            self.width = int(dims[0])
            self.height = int(dims[1])
            self.origGrid = [[Level.EMPTY for i in range(self.width)] for j in range(self.height)]

            # read the actual grid
            for i in range(self.height):
                line = f.readline()
                for j in range(self.width):
                    if line[j] == '#':
                        self.origGrid[i][j] = Level.BLOCKED
                    elif line[j] == ' ':
                        self.origGrid[i][j] = Level.PASSABLE
                    else:
                        self.origGrid[i][j] = Level.EMPTY
            self.grid = copy.deepcopy(self.origGrid)
            
            # read number of placeable arrows
            arrows = f.readline().split(' ')
            self.origNumArrows = [int(x) for x in arrows]
            self.numArrows = copy.deepcopy(self.origNumArrows)
            assert(len(self.numArrows) == 4)
            self.game.update_arrows(self.numArrows)
            
            # read positions of animals
            numAnimals = int(f.readline())
            self.animals = []
            for i in range(numAnimals):
                params = f.readline().split(' ')
                self.animals.append((int(params[0]), int(params[1]), int(params[2]), int(params[3])))

            # read level title
            self.title = f.readline()[:-1]  # get rid of trailing newline
            # read description
            self.description = ' '.join(f.readlines()).replace('\n', '')
            
            self.offset = [0,0]
            
        with open(filename, 'rb') as f:
            # compile md5 checksum as level identifier
            m = hashlib.md5()
            m.update(f.read())
            self.hash = m.hexdigest()
            
        # load tileset; seed random number generator from level hash
        random.seed(self.hash)
        self.load_tile_set()
        
            
    def clear(self):
        self.numArrows = copy.deepcopy(self.origNumArrows)
        self.grid = copy.deepcopy(self.origGrid)
        self.game.update_arrows(self.numArrows)
        
        
    def get_tile(self, x, y):
        return self.grid[y][x]
        
        
    def get_neighbour(self, x, y, dir):
        if dir == 0:
            nx = x - 1
            ny = y
        elif dir == 1:
            nx = x + 1
            ny = y
        elif dir == 2:
            nx = x
            ny = y - 1
        else:
            nx = x
            ny = y + 1
        nx = nx % self.width  # wrap around ends of field
        ny = ny % self.height
        if self.grid[ny][nx] == Level.EMPTY:  # skip empty tiles
            nx, ny = self.get_neighbour(nx, ny, dir)
        return nx, ny
            

    def calc_offset(self):
        # we use a rendering offset for level scrolling which refers to the
        # middle of the level; figure out actual offset from the top left corner
        # of our rendering area
        offx = self.renderRect[0] + self.renderRect[2]//2 + self.offset[0]
        offy = self.renderRect[1] + self.renderRect[3]//2 + self.offset[1]
        offx = offx - int((self.width/2) * Level.TILESIZE)
        offy = offy - int((self.height/2) * Level.TILESIZE)
        return (int(offx), int(offy))
        
        
    def get_tile_coords(self, x, y):
        offx, offy = self.calc_offset()
        return (offx + x*Level.TILESIZE, offy + y*Level.TILESIZE)

        
    def mouse_moved(self, pos):
        offx, offy = self.calc_offset()
        tilex = (pos[0]-offx) // Level.TILESIZE
        tiley = (pos[1]-offy) // Level.TILESIZE
        if (tilex >= 0 and tilex < self.width and tiley >= 0 and tiley < self.height and
            not self.grid[tiley][tilex] in (Level.BLOCKED, Level.EMPTY)):
            self.highlightX = tilex
            self.highlightY = tiley
        else:
            self.highlightX = -1
            self.highlightY = -1
            
            
    def mouse_down(self, pos, arrow):
        self.mouse_moved(pos)
        if self.highlightX == -1: return
        cur = self.grid[self.highlightY][self.highlightX]
        if cur >= Level.ARROW_LEFT:
            self.grid[self.highlightY][self.highlightX] = Level.PASSABLE
            self.numArrows[cur-Level.ARROW_LEFT] += 1
            
        if arrow != 4:
            if self.numArrows[arrow] <= 0: return
            # place new arrow on that particular tile
            self.grid[self.highlightY][self.highlightX] = arrow + Level.ARROW_LEFT
            self.numArrows[arrow] -= 1
            self.sound.play()
            
        self.game.update_arrows(self.numArrows)
        
        
    def render(self, display):
        offx, offy = self.calc_offset()
        # blit tile images to display
        for i in range(self.height):
            posy = offy + i*Level.TILESIZE
            for j in range(self.width):
                posx = offx + j*Level.TILESIZE
                if self.grid[i][j] == Level.EMPTY: continue
                display.blit(self.images[self.grid[i][j]][self.rand[i][j]], (posx, posy))
                if self.highlightX == j and self.highlightY == i:
                    # highlight current tile due to mouseover
                    pygame.draw.rect(display, (0,255,0,20), (posx,posy,Level.TILESIZE,Level.TILESIZE), 1)
                